@@ -220,6 +220,7 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             'LDRN': chebyshev.bases.DirichletNeumann,
             'LNRD': chebyshev.bases.NeumannDirichlet,
             'RD': chebyshev.bases.UpperDirichlet,
+            'LD': chebyshev.bases.LowerDirichlet,
             'RDRN': chebyshev.bases.UpperDirichletNeumann,
             'LDLNRDRN': chebyshev.bases.ShenBiharmonic
         }
@@ -250,11 +251,13 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             'LNRN': legendre.bases.ShenNeumann,
             'LDRN': legendre.bases.DirichletNeumann,
             'LNRD': legendre.bases.NeumannDirichlet,
+            'LD': legendre.bases.LowerDirichlet,
             'RD': legendre.bases.UpperDirichlet,
             'RDRN': legendre.bases.UpperDirichletNeumann,
             'LNRDRN': legendre.bases.ShenBiPolar0,
             'LDLNRDRN': legendre.bases.ShenBiharmonic,
-            'LDLNRN2RN3': legendre.bases.BeamFixedFree
+            'LDLNRN2RN3': legendre.bases.BeamFixedFree,
+            'LDLNRD': legendre.bases.DirichletNeumannDirichlet
         }
 
         if quad is not None:
@@ -340,7 +343,7 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             if len(bc) == 2:
                 B = jacobi.bases.ShenDirichlet
             else:
-                assert np.all([abs(bci)<1e-12 for bci in bc])
+                assert np.all([abs(bci) < 1e-12 for bci in bc])
                 B = jacobi.bases.ShenBiharmonic
 
         elif isinstance(bc, str):
@@ -899,7 +902,7 @@ class Expr:
                 match = []
                 if len(tms[-1]) > 0:
                     match = np.where((np.array(term) == np.array(tms[-1])).prod(axis=1) == 1)[0]
-                    matchi = np.where( ind == np.array(inds[-1]))[0]
+                    matchi = np.where(ind == np.array(inds[-1]))[0]
                     match = np.intersect1d(match, matchi)
                 if len(match) > 0:
                     k = match[0]
@@ -1152,9 +1155,13 @@ class ShenfunBaseArray(DistArray):
 
             elif hasattr(buffer, 'free_symbols'):
                 # Evaluate sympy function on entire mesh
-                x = buffer.free_symbols.pop()
-                buffer = sp.lambdify(x, buffer)
-                buf = buffer(space.mesh()).astype(space.forward.input_array.dtype)
+                syms = buffer.free_symbols
+                if len(syms) > 0:
+                    x = syms.pop()
+                    buffer = sp.lambdify(x, buffer)
+                    buf = buffer(space.mesh()).astype(space.forward.input_array.dtype)
+                else:
+                    buf = buffer
                 buffer = Array(space)
                 buffer.v[:] = buf
                 if cls.__name__ == 'Function':
@@ -1302,6 +1309,13 @@ class ShenfunBaseArray(DistArray):
 
         return np.ndarray.__getitem__(self.v, i)
 
+    def __getattribute__(self, name):
+        names = ('all', 'any', 'argmax', 'argmin', 'argpartition', 'argsort', 'byteswap', 'choose', 'clip', 'compress', 'cumprod', 'cumsum', 'diagonal', 'flatten', 'getfield', 'item', 'max', 'mean', 'min', 'newbyteorder', 'nonzero', 'partition', 'prod', 'ptp', 'put', 'ravel', 'repeat', 'resize', 'round', 'searchsorted', 'setflags', 'sort', 'squeeze', 'std', 'sum', 'swapaxes', 'take', 'tobytes', 'tofile', 'tolist', 'tostring', 'trace', 'transpose', 'var', 'view', 'ndim', 'flags', 'strides', 'itemsize', 'size', 'nbytes', 'dtype', 'flat', 'ctypes')
+        if name in names:
+            a = object.__getattribute__(self, '__array__')()
+            return np.ndarray.__getattribute__(a, name)
+        return object.__getattribute__(self, name)
+
     def dim(self):
         return self.function_space().dim()
 
