@@ -3,6 +3,7 @@ This module contains the inner function that computes the
 weighted inner product.
 """
 from numbers import Number
+from copy import copy
 import numpy as np
 import sympy as sp
 from shenfun.spectralbase import inner_product, SpectralBase, MixedFunctionSpace
@@ -18,13 +19,13 @@ __all__ = ('inner',)
 
 def inner(expr0, expr1, output_array=None, level=0):
     r"""
-    Return (weighted or unweighted) discrete inner product of kind
+    Return (weighted or unweighted) discrete inner product
 
     .. math::
 
         (f, g)_w^N = \sum_{i\in\mathcal{I}}f(x_i) \overline{g}(x_i) w_i \approx \int_{\Omega} f\, \overline{g}\, w\, dx
 
-    where :math:`\mathcal{I}=0, 1, \ldots, N, N \in \mathbb{Z}^+`, :math:`f`
+    where :math:`\mathcal{I}=0, 1, \ldots, N-1, N \in \mathbb{Z}^+`, :math:`f`
     is a number or an expression linear in a :class:`.TestFunction`,
     and :math:`g` is an expression that is linear in :class:`.TrialFunction`
     or :class:`.Function`, or it is simply an :class:`.Array` (a solution interpolated on the
@@ -326,7 +327,6 @@ def inner(expr0, expr1, output_array=None, level=0):
                         continue
                     for dv in split(dV):
                         sc = dv['coeff']
-                        scb = dv['coeff']
                         M = []
                         DM = []
                         for i, (a, b) in enumerate(zip(b0, b1)): # Third index, one inner for each dimension
@@ -340,17 +340,15 @@ def inner(expr0, expr1, output_array=None, level=0):
                             if len(AA) == 0:
                                 AA = Identity(AA.shape, scale=0)
 
-                            M.append(AA)
                             if not abs(AA.scale-1.) < 1e-8:
-                                sc *= AA.scale
-                                AA.scale = 1.0
+                                AA.incorporate_scale()
+                            M.append(AA)
 
                             if ts.has_nonhomogeneous_bcs:
                                 tsc = ts.get_bc_basis()
                                 BB = inner_product((tt, a), (tsc, b), msi)
                                 if not abs(BB.scale-1.) < 1e-8:
-                                    scb *= BB.scale
-                                    BB.scale = 1.0
+                                    BB.incorporate_scale()
                                 if BB:
                                     DM.append(BB)
                                     has_bcs = True
@@ -360,7 +358,6 @@ def inner(expr0, expr1, output_array=None, level=0):
                                 DM.append(AA)
 
                         sc = tt.broadcast_to_ndims(np.array([sc]))
-                        scb = tt.broadcast_to_ndims(np.array([scb]))
                         if len(M) == 1: # 1D case
                             M[0].global_index = (test_ind[test_j], trial_ind[trial_j])
                             M[0].scale = sc[0]
@@ -371,19 +368,19 @@ def inner(expr0, expr1, output_array=None, level=0):
                         if has_bcs:
                             if len(DM) == 1: # 1D case
                                 DM[0].global_index = (test_ind[test_j], trial_ind[trial_j])
-                                DM[0].scale = scb
+                                DM[0].scale = sc
                                 DM[0].mixedbase = testspace
                                 A.append(DM[0])
                             else:
                                 if len(trial_sp.get_nonhomogeneous_axes()) == 1:
-                                    A.append(TPMatrix(DM, test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                    A.append(TPMatrix(DM, test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
                                 elif len(trial_sp.get_nonhomogeneous_axes()) == 2:
                                     if DM[1] != 0:
-                                        A.append(TPMatrix([M[0], DM[1]], test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                        A.append(TPMatrix([M[0], DM[1]], test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
                                     if DM[0] != 0:
-                                        A.append(TPMatrix([DM[0], M[1]], test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                        A.append(TPMatrix([DM[0], M[1]], test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
                                     if DM[0] != 0 and DM[1] != 0:
-                                        A.append(TPMatrix(DM, test_sp, trial_sp, scb, (test_ind[test_j], trial_ind[trial_j]), testspace))
+                                        A.append(TPMatrix(DM, test_sp, trial_sp, sc, (test_ind[test_j], trial_ind[trial_j]), testspace))
 
     # At this point A contains all matrices of the form. The length of A is
     # the number of inner products. For each index into A there are ndim 1D
