@@ -29,14 +29,14 @@ class BHHmat(SpectralMatrix):
         assert isinstance(trial[0], H)
         SpectralMatrix.__init__(self, {0:1}, test, trial)
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, constraints=()):
         if u is not None:
             u[:] = b
-            u /= self.scale
+            u /= (self.scale*self[0])
             return u
 
         else:
-            b /= self.scale
+            b /= (self.scale*self[0])
             return b
 
     def matvec(self, v, c, format='python', axis=0):
@@ -44,7 +44,7 @@ class BHHmat(SpectralMatrix):
         ss = [slice(None)]*len(v.shape)
         ss[self.axis] = slice(0, M)
         c[tuple(ss)] = v
-        self.scale_array(c)
+        self.scale_array(c, self.scale*self[0])
         return c
 
 
@@ -74,7 +74,9 @@ class AHHmat(SpectralMatrix):
         d[0][-1] = (N-1)/2.
         d[-2] = d[2]
         SpectralMatrix.__init__(self, d, test, trial)
-        self.solve = TDMA(self)
+
+    def get_solver(self):
+        return TDMA
 
     def matvec(self, v, c, format='cython', axis=0):
         N, M = self.shape
@@ -82,15 +84,15 @@ class AHHmat(SpectralMatrix):
         if format == 'cython' and v.ndim == 3:
             ld = self[-2]*np.ones(M-2)
             Matvec.Tridiagonal_matvec3D_ptr(v, c, ld, self[0], ld, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale)
         elif format == 'cython' and v.ndim == 2:
             ld = self[-2]*np.ones(M-2)
             Matvec.Tridiagonal_matvec2D_ptr(v, c, ld, self[0], ld, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale)
         elif format == 'cython' and v.ndim == 1:
             ld = self[-2]*np.ones(M-2)
             Matvec.Tridiagonal_matvec(v, c, ld, self[0], ld)
-            self.scale_array(c)
+            self.scale_array(c, self.scale)
         elif format == 'self':
             if axis > 0:
                 v = np.moveaxis(v, axis, 0)
@@ -102,7 +104,7 @@ class AHHmat(SpectralMatrix):
             if axis > 0:
                 v = np.moveaxis(v, 0, axis)
                 c = np.moveaxis(c, 0, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale)
 
         else:
             c = super(AHHmat, self).matvec(v, c, format=format, axis=axis)
