@@ -69,9 +69,8 @@ import functools
 import numpy as np
 import sympy as sp
 from shenfun.matrixbase import SpectralMatrix
-from shenfun.la import TDMA as neumann_TDMA
 from shenfun.optimization import cython
-from .la import TDMA
+from shenfun.la import TDMA, PDMA
 from . import bases
 
 # Short names for instances of bases
@@ -118,7 +117,7 @@ class BLLmat(SpectralMatrix):
             d[0][-1] = 2./(N-1)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, constraints=()):
         s = self.trialfunction[0].slice()
         if u is None:
             u = b
@@ -166,8 +165,9 @@ class BSDSDmat(SpectralMatrix):
 
         d[2] = d[-2]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
-        self.solve = TDMA(self)
 
+    def get_solver(self):
+        return TDMA
 
 class BSNSNmat(SpectralMatrix):
     r"""Mass matrix for inner product
@@ -218,7 +218,6 @@ class BSBSBmat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, measure=1):
-        from shenfun.la import PDMA
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
@@ -234,7 +233,9 @@ class BSBSBmat(SpectralMatrix):
         d[-2] = d[2]
         d[-4] = d[4]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
-        self.solve = PDMA(self)
+
+    def get_solver(self):
+        return PDMA
 
 class BBFBFmat(SpectralMatrix):
     r"""Mass matrix for inner product
@@ -403,7 +404,7 @@ class ASDSDmat(SpectralMatrix):
             d = {0: 1}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, constraints=()):
         N = self.shape[0] + 2
         assert N == b.shape[axis]
         s = self.trialfunction[0].slice()
@@ -439,7 +440,7 @@ class ASDSDmat(SpectralMatrix):
             ss[axis] = s
             ss = tuple(ss)
             u[ss] = b[ss]
-            u /= self.scale
+            u /= (self.scale*self[0])
             self.testfunction[0].bc.set_boundary_dofs(u, True)
 
         return u
@@ -471,7 +472,7 @@ class ASNSNmat(SpectralMatrix):
         d = {0: d0*alpha*(k+0.5)*((k+2)*(k+3)-k*(k+1))}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, constraints=()):
         N = self.shape[0] + 2
         assert N == b.shape[axis]
         s = self.trialfunction[0].slice()
@@ -495,7 +496,8 @@ class ASNSNmat(SpectralMatrix):
         us[:] = bs*d[tuple(sl)]
         u /= self.scale
         self.testfunction[0].bc.set_boundary_dofs(u, True)
-        u[0] = self.testfunction[0].mean/(2/self.testfunction[0].domain_factor())
+        if self.testfunction[0].use_fixed_gauge:
+            u[0] = self.testfunction[0].mean/(2/self.testfunction[0].domain_factor())
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
             if u is not b:
@@ -555,7 +557,7 @@ class ADNDNmat(SpectralMatrix):
         d = {0: ((k+1)/(k+2))**2*((k+2)*(k+3)- k*(k+1))}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, constraints=()):
         N = self.shape[0] + 2
         assert N == b.shape[axis]
         s = self.trialfunction[0].slice()
@@ -613,7 +615,7 @@ class SBFBFmat(SpectralMatrix):
         d = {0: f4*(k+2.5)*((k+4)*(k+5)-(k+2)*(k+3))*((k+2)*(k+3)-k*(k+1))}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
-    def solve(self, b, u=None, axis=0):
+    def solve(self, b, u=None, axis=0, constraints=()):
         N = self.shape[0] + 4
         assert N == b.shape[axis]
         s = self.trialfunction[0].slice()
@@ -667,10 +669,16 @@ class GLLmat(SpectralMatrix):
         assert isinstance(trial[0], L)
         N = test[0].N
         k = np.arange(N, dtype=float)
-        d = {}
-        for j in range(2, N, 2):
-            jj = j if trial[1] else -j
-            d[jj] = (k[:-j]+0.5)*(k[j:]*(k[j:]+1) - k[:-j]*(k[:-j]+1))*2./(2*k[:-j]+1)
+        self._keyscale = 1
+        def _getkey(i):
+            j = abs(i)
+            return self._keyscale*((k[:-j]+0.5)*(k[j:]*(k[j:]+1) - k[:-j]*(k[:-j]+1))*2./(2*k[:-j]+1))
+
+        if trial[1]:
+            d = dict.fromkeys(np.arange(2, N, 2), _getkey)
+        else:
+            d = dict.fromkeys(-np.arange(2, N, 2), _getkey)
+
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self._matvec_methods += ['cython']
 
@@ -679,13 +687,13 @@ class GLLmat(SpectralMatrix):
         trial = self.trialfunction[1]
         if format == 'cython' and v.ndim == 3 and trial:
             cython.Matvec.GLL_matvec3D_ptr(v, c, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale*self._keyscale)
         elif format == 'cython' and v.ndim == 2 and trial:
             cython.Matvec.GLL_matvec2D_ptr(v, c, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale*self._keyscale)
         elif format == 'cython' and v.ndim == 1 and trial:
             cython.Matvec.GLL_matvec(v, c)
-            self.scale_array(c)
+            self.scale_array(c, self.scale*self._keyscale)
         else:
             c = super(GLLmat, self).matvec(v, c, format=format, axis=axis)
         return c
@@ -735,9 +743,11 @@ class CLLmat(SpectralMatrix):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], L)
         N = test[0].N
-        d = {}
-        for i in range(1, N, 2):
-            d[i] = 2
+        self._keyscale = 1
+        def _getkey(i):
+            return 2*self._keyscale
+
+        d = dict.fromkeys(np.arange(1, N, 2), _getkey)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self._matvec_methods += ['cython', 'self']
 
@@ -754,17 +764,17 @@ class CLLmat(SpectralMatrix):
             if axis > 0:
                 c = np.moveaxis(c, 0, axis)
                 v = np.moveaxis(v, 0, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale*self._keyscale)
 
         elif format == 'cython' and v.ndim == 3:
             cython.Matvec.CLL_matvec3D_ptr(v, c, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale*self._keyscale)
         elif format == 'cython' and v.ndim == 2:
             cython.Matvec.CLL_matvec2D_ptr(v, c, axis)
-            self.scale_array(c)
+            self.scale_array(c, self.scale*self._keyscale)
         elif format == 'cython' and v.ndim == 1:
             cython.Matvec.CLL_matvec(v, c)
-            self.scale_array(c)
+            self.scale_array(c, self.scale*self._keyscale)
         else:
             c = super(CLLmat, self).matvec(v, c, format=format, axis=axis)
         return c
@@ -789,9 +799,11 @@ class CLLmatT(SpectralMatrix):
         assert isinstance(test[0], L)
         assert isinstance(trial[0], L)
         N = test[0].N
-        d = {}
-        for i in range(-1, -N, -2):
-            d[i] = 2
+        self._keyscale = 1
+        def _getkey(i):
+            return 2*self._keyscale
+
+        d = dict.fromkeys(-np.arange(1, N, 2), _getkey)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
@@ -872,8 +884,35 @@ class CSDSDmat(SpectralMatrix):
         d = {-1: -2, 1: 2}
         if trial[0].is_scaled():
             k = np.arange(N-2, dtype=float)
-            d[-1] = -2. / np.sqrt(4*k[:-1]+6)
-            d[1] = 2. / np.sqrt(4*k[:-1]+6)
+            d[-1] = -2/np.sqrt(4*k[:-1]+6)
+            d[1] = 2/np.sqrt(4*k[:-1]+6)
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class CSDSDTmat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        C_{kj} = (\psi_j, \psi'_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Shen Legendre Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], SD)
+        assert isinstance(trial[0], SD)
+        N = test[0].N
+        d = {-1: 2, 1: -2}
+        if trial[0].is_scaled():
+            k = np.arange(N-2, dtype=float)
+            d[-1] = 2/np.sqrt(4*k[:-1]+6)
+            d[1] = -2/np.sqrt(4*k[:-1]+6)
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
