@@ -42,8 +42,8 @@ def preLU(A, B, neumann=False):
         j2 = k**2
         j2[0] = 1
         j2 = 1/j2
-        j2[0] = 0
         B_0 *= j2
+        j2[0] = 0
         A_0 *= j2
         A_2 *= j2[2:]
         B_2 *= j2[2:]
@@ -69,7 +69,8 @@ def preLU(A, B, neumann=False):
 def LU_Helmholtz_1D(A_0, A_2, A_4, B_m2, B_0, B_2, A_scale, B_scale, neumann, d0, d1, d2, L):
     N = A_0.shape[0]
     if neumann:
-        A_0[0] = np.pi/A_scale
+        if abs(B_scale) < 1e-8:
+            A_0[0] = 1/A_scale
 
     d0[0] = A_scale*A_0[0] + B_scale*B_0[0]
     d0[1] = A_scale*A_0[1] + B_scale*B_0[1]
@@ -177,7 +178,8 @@ def Solve_Helmholtz_1D(fk, u_hat, neumann, d0, d1, d2, L, y):
         u_hat[i] /= d0[i]
 
     if neumann:
-        u_hat[0] = 0.0
+        if (d0[0]-1.0)*(d0[0]-1.0) < 1e-16:
+            u_hat[0] = 0.0
         for i in range(1, N):
             u_hat[i] /= (i*i)
 
@@ -250,8 +252,8 @@ def Helmholtz_matvec3D(v, b, alfa, beta, dd, ud, bd, axis):
     elif axis == 1:
         for i in range(v.shape[0]):
             for k in range(v.shape[2]):
-               Helmholtz_matvec1D(v[i, :, k], b[i, :, k], alfa[i, 0, k],
-                                  beta[i, 0, k], dd, ud, bd)
+                Helmholtz_matvec1D(v[i, :, k], b[i, :, k], alfa[i, 0, k],
+                                   beta[i, 0, k], dd, ud, bd)
     elif axis == 2:
         for i in range(v.shape[0]):
             for j in range(v.shape[1]):
@@ -347,17 +349,17 @@ def Helmholtz_Neumann_matvec3D(v, b, alfa, beta, dd, ud, bl, bd, bu, axis):
         for j in range(v.shape[1]):
             for k in range(v.shape[2]):
                 Helmholtz_Neumann_matvec1D(v[:, j, k], b[:, j, k], alfa[0, j, k],
-                                   beta[0, j, k], dd, ud, bl, bd, bu)
+                                           beta[0, j, k], dd, ud, bl, bd, bu)
     elif axis == 1:
         for i in range(v.shape[0]):
             for k in range(v.shape[2]):
                Helmholtz_Neumann_matvec1D(v[i, :, k], b[i, :, k], alfa[i, 0, k],
-                                  beta[i, 0, k], dd, ud, bl, bd, bu)
+                                          beta[i, 0, k], dd, ud, bl, bd, bu)
     elif axis == 2:
         for i in range(v.shape[0]):
             for j in range(v.shape[1]):
                 Helmholtz_Neumann_matvec1D(v[i, j], b[i, j], alfa[i, j, 0],
-                                   beta[i, j, 0], dd, ud, bl, bd, bu)
+                                           beta[i, j, 0], dd, ud, bl, bd, bu)
 
 @nb.jit(nopython=True, fastmath=True, cache=True)
 def Helmholtz_Neumann_matvec2D(v, b, alfa, beta, dd, ud, bl, bd, bu, axis):
@@ -412,15 +414,16 @@ def Poisson_Solve_ADD_1D(d, d1, scale, b, u):
     if not abs(scale-1) < 1e-8:
         for k in range(N):
             u[k] /= scale
+    return u
 
 @nb.jit(nopython=True, fastmath=True, cache=True)
 def Poisson_Solve_ADD_2D(d, d1, scale, b, u, axis):
     if axis == 0:
         for j in range(u.shape[1]):
-            Poisson_Solve_ADD_1D(d, d1, scale, b[:, j], u[:, j])
+            u[:, j] = Poisson_Solve_ADD_1D(d, d1, scale, b[:, j], u[:, j])
     elif axis == 1:
         for i in range(u.shape[0]):
-            Poisson_Solve_ADD_1D(d, d1, scale, b[:, j], u[:, j])
+            u[i] = Poisson_Solve_ADD_1D(d, d1, scale, b[i], u[i])
     return u
 
 @nb.jit(nopython=True, fastmath=True, cache=True)
@@ -428,30 +431,36 @@ def Poisson_Solve_ADD_3D(d, d1, scale, b, u, axis):
     if axis == 0:
         for j in range(u.shape[1]):
             for k in range(u.shape[2]):
-                Poisson_Solve_ADD_1D(d, d1, scale, b[:, j, k], u[:, j, k])
+                u[:, j, k] = Poisson_Solve_ADD_1D(d, d1, scale, b[:, j, k], u[:, j, k])
 
     elif axis == 1:
         for i in range(u.shape[0]):
             for k in range(u.shape[2]):
-                Poisson_Solve_ADD_1D(d, d1, scale, b[i, :, k], u[i, :, k])
+                u[i, :, k] = Poisson_Solve_ADD_1D(d, d1, scale, b[i, :, k], u[i, :, k])
+
+    elif axis == 2:
+        for i in range(u.shape[0]):
+            for j in range(u.shape[1]):
+                u[i, j] = Poisson_Solve_ADD_1D(d, d1, scale, b[i, j], u[i, j])
+    return u
 
 @nb.jit(nopython=True, fastmath=True, cache=True)
 def ANN_matvec1D(v, c, d0, d2, scale=1):
-        N = v.shape[0]-2
-        c[N-1] = d0[N-1]*v[N-1]
-        c[N-2] = d0[N-2]*v[N-2]
-        s0 = 0
-        s1 = 0
-        for k in range(N-3, 0, -1):
-            c[k] = d0[k]*v[k]
-            if k % 2 == 0:
-                s0 += v[k+2]*(k+2)**2
-                c[k] += s0*d2[k]/(k+2)**2
-            else:
-                s1 += v[k+2]*(k+2)**2
-                c[k] += s1*d2[k]/(k+2)**2
-        if scale != 1:
-            c /= scale
+    N = v.shape[0]-2
+    c[N-1] = d0[N-1]*v[N-1]
+    c[N-2] = d0[N-2]*v[N-2]
+    s0 = 0
+    s1 = 0
+    for k in range(N-3, -1, -1):
+        c[k] = d0[k]*v[k]
+        if (k % 2) == 0:
+            s0 += v[k+2]*(k+2)**2
+            c[k] += s0*d2[k]/(k+2)**2
+        else:
+            s1 += v[k+2]*(k+2)**2
+            c[k] += s1*d2[k]/(k+2)**2
+    if scale != 1:
+        c /= scale
 
 @nb.jit(nopython=True, fastmath=True, cache=True)
 def ANN_matvec2D(v, b, d0, d2, scale, axis):
@@ -471,7 +480,7 @@ def ANN_matvec3D(v, b, d0, d2, scale, axis):
     elif axis == 1:
         for i in range(v.shape[0]):
             for k in range(v.shape[2]):
-               ANN_matvec1D(v[i, :, k], b[i, :, k], d0, d2, scale)
+                ANN_matvec1D(v[i, :, k], b[i, :, k], d0, d2, scale)
     elif axis == 2:
         for i in range(v.shape[0]):
             for j in range(v.shape[1]):
