@@ -231,6 +231,9 @@ class SpectralBase:
 
         self._evaluate_scalar_product(fast_transform=fast_transform)
 
+        self._truncation_forward(self.scalar_product.tmp_array,
+                                 self.scalar_product.output_array)
+
         if output_array is not None:
             output_array[...] = self.scalar_product.output_array
             return output_array
@@ -256,8 +259,6 @@ class SpectralBase:
 
         """
         self.scalar_product(input_array, fast_transform=fast_transform)
-        self._truncation_forward(self.forward.tmp_array,
-                                 self.forward.output_array)
         if self.bc:
             self.bc.add_mass_rhs(self.forward.output_array)
         self.apply_inverse_mass(self.forward.output_array)
@@ -312,7 +313,8 @@ class SpectralBase:
         return self.backward.output_array
 
     def vandermonde(self, x):
-        r"""Return Vandermonde matrix based on the primary basis of the family.
+        r"""Return Vandermonde matrix based on the primary (orthogonal) basis
+        of the family.
 
         Evaluates basis :math:`\psi_k(x)` for all wavenumbers, and all ``x``.
         Returned Vandermonde matrix is an N x M matrix with N the length of
@@ -334,7 +336,7 @@ class SpectralBase:
 
         Note
         ----
-        This function returns a matrix of evaluated primary basis functions for
+        This function returns a matrix of evaluated orthogonal basis functions for
         either family. That is, it is using either pure Chebyshev, Legendre or
         Fourier exponentials. The true Vandermonde matrix of a basis is obtained
         through :meth:`.SpectralBase.evaluate_basis_all`.
@@ -511,9 +513,9 @@ class SpectralBase:
         self.scalar_product.output_array
 
         """
-        # This is the slow Vandermone type implementation
+        # This is the slow Vandermonde type implementation
         input_array = self.scalar_product.input_array
-        output_array = self.scalar_product.output_array
+        output_array = self.scalar_product.tmp_array
         M = self.shape(False)
         weights = self.points_and_weights(M)[1]
         P = self.evaluate_basis_all(argument=0)
@@ -541,7 +543,6 @@ class SpectralBase:
             array
 
         """
-        #assert self.N == array.shape[self.axis]
         if self._mass is None:
             B = self.get_mass_matrix()
             self._mass = B((self, 0), (self, 0))
@@ -601,67 +602,40 @@ class SpectralBase:
         plan_fwd = self._xfftn_fwd
         plan_bck = self._xfftn_bck
 
-        if 'builders' in self._xfftn_fwd.__module__: #pragma: no cover
+        opts = dict(
+            overwrite_input='FFTW_DESTROY_INPUT',
+            planner_effort='FFTW_MEASURE',
+            threads=1,
+        )
+        opts.update(options)
+        flags = (fftw.flag_dict[opts['planner_effort']],
+                 fftw.flag_dict[opts['overwrite_input']])
+        threads = opts['threads']
 
-            opts = dict(
-                avoid_copy=True,
-                overwrite_input=True,
-                auto_align_input=True,
-                auto_contiguous=True,
-                planner_effort='FFTW_MEASURE',
-                threads=1,
-                backward=False,
-            )
-            opts.update(options)
+        n = (shape[axis],)
+        U = fftw.aligned(shape, dtype=dtype)
+        xfftn_fwd = plan_fwd(U, n, (axis,), threads=threads, flags=flags)
+        V = xfftn_fwd.output_array
 
-            n = shape[axis]
-            U = fftw.aligned(shape, dtype=dtype)
-            xfftn_fwd = plan_fwd(U, n=n, axis=axis, **opts)
-            V = xfftn_fwd.output_array
-            xfftn_bck = plan_bck(V, n=n, axis=axis, **opts)
-            V.fill(0)
-            U.fill(0)
-
-            xfftn_fwd.update_arrays(U, V)
-            xfftn_bck.update_arrays(V, U)
-            self._M = 1./np.prod(np.take(shape, axis))
-
-        else:
-            opts = dict(
-                overwrite_input='FFTW_DESTROY_INPUT',
-                planner_effort='FFTW_MEASURE',
-                threads=1,
-            )
-            opts.update(options)
-            flags = (fftw.flag_dict[opts['planner_effort']],
-                     fftw.flag_dict[opts['overwrite_input']])
-            threads = opts['threads']
-
-            n = (shape[axis],)
-            U = fftw.aligned(shape, dtype=dtype)
-            xfftn_fwd = plan_fwd(U, n, (axis,), threads=threads, flags=flags)
-            V = xfftn_fwd.output_array
-
-            if np.issubdtype(dtype, floating):
-                flags = (fftw.flag_dict[opts['planner_effort']],)
-
-            xfftn_bck = plan_bck(V, n, (axis,), threads=threads, flags=flags, output_array=U)
-            V.fill(0)
-            U.fill(0)
-            self._M = xfftn_fwd.get_normalization()
+        if np.issubdtype(dtype, np.floating):
+            flags = (fftw.flag_dict[opts['planner_effort']],)
 
+        xfftn_bck = plan_bck(V, n, (axis,), threads=threads, flags=flags, output_array=U)
+        V.fill(0)
+        U.fill(0)
+        self._M = xfftn_fwd.get_normalization()
         self.axis = axis
 
         if self.padding_factor != 1:
             trunc_array = self._get_truncarray(shape, V.dtype)
+            self.scalar_product = Transform(self.scalar_product, xfftn_fwd, U, V, trunc_array)
             self.forward = Transform(self.forward, xfftn_fwd, U, V, trunc_array)
             self.backward = Transform(self.backward, xfftn_bck, trunc_array, V, U)
         else:
+            self.scalar_product = Transform(self.scalar_product, xfftn_fwd, U, V, V)
             self.forward = Transform(self.forward, xfftn_fwd, U, V, V)
             self.backward = Transform(self.backward, xfftn_bck, V, V, U)
 
-        # scalar_product is not padded, just the forward/backward
-        self.scalar_product = Transform(self.scalar_product, xfftn_fwd, U, V, V)
         self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
@@ -970,66 +944,118 @@ class SpectralBase:
             raise NotImplementedError
         return array
 
-    def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
+    def get_dealiased(self, padding_factor=1.5, dealias_direct=False, **kwargs):
         """Return space (otherwise as self) to be used for dealiasing
 
         Parameters
         ----------
         padding_factor : float, optional
-            Create output array of shape padding_factor time non-padded shape
+            Create output array of shape padding_factor times non-padded shape
         dealias_direct : bool, optional
             If True, set upper 2/3 of wavenumbers to zero before backward transform.
             Cannot be used together with padding_factor different than 1.
+        kwargs : keyword arguments
+            Any other keyword arguments used in the creation of the bases.
 
         Returns
         -------
         SpectralBase
             The space to be used for dealiasing
         """
-        return self.__class__(self.N,
-                              quad=self.quad,
-                              domain=self.domain,
-                              dtype=self.dtype,
-                              padding_factor=padding_factor,
-                              dealias_direct=dealias_direct,
-                              coordinates=self.coors.coordinates)
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype,
+                 padding_factor=padding_factor,
+                 dealias_direct=dealias_direct,
+                 coordinates=self.coors.coordinates)
+        if hasattr(self.bc, 'bc'):
+            d['bc'] = tuple(self.bc.bc)
+        if hasattr(self, '_scaled'):
+            d['scaled'] = self._scaled
+        for key in ('alpha', 'beta', 'mean'):
+            if hasattr(self, key):
+                d[key] = object.__getattribute__(self, key)
+        d.update(kwargs)
+        return self.__class__(self.N, **d)
 
-    def get_refined(self, N):
+    def get_refined(self, N, **kwargs):
         """Return space (otherwise as self) with N quadrature points
 
         Parameters
         ----------
         N : int
             The number of quadrature points for returned space
+        kwargs : keyword arguments
+            Any other keyword arguments used in the creation of the bases.
 
         Returns
         -------
         SpectralBase
             A new space with new number of quadrature points, otherwise as self.
         """
-        return self.__class__(N,
-                              quad=self.quad,
-                              domain=self.domain,
-                              dtype=self.dtype,
-                              padding_factor=self.padding_factor,
-                              dealias_direct=self.dealias_direct,
-                              coordinates=self.coors.coordinates)
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype,
+                 padding_factor=self.padding_factor,
+                 dealias_direct=self.dealias_direct,
+                 coordinates=self.coors.coordinates)
+        if hasattr(self.bc, 'bc'):
+            d['bc'] = tuple(self.bc.bc)
+        if hasattr(self, '_scaled'):
+            d['scaled'] = self._scaled
+        for key in ('alpha', 'beta', 'mean'):
+            if hasattr(self, key):
+                d[key] = object.__getattribute__(self, key)
+        d.update(kwargs)
+        return self.__class__(N, **d)
 
-    def get_unplanned(self):
+    def get_unplanned(self, **kwargs):
         """Return unplanned space (otherwise as self)
 
+        Parameters
+        ----------
+        kwargs : keyword arguments, optional
+            Any keyword argument used in the creation of the unplanned
+            space. Could be any one of
+
+            - quad
+            - domain
+            - dtype
+            - padding_factor
+            - dealias_direct
+            - coorrdinates
+            - mean
+            - bc
+            - scaled
+
+            Not all will be applicable for all spaces.
+
         Returns
         -------
         SpectralBase
             Space not planned for a :class:`.TensorProductSpace`
+
+        Note
+        ----
+        Default is to return a basic space not intended for dealiasing.
+        So the `planning_factor` is set to 1 and `dealias_direct` is
+        False. This can be overruled by keyword arguments.
         """
-        return self.__class__(self.N,
-                              quad=self.quad,
-                              domain=self.domain,
-                              dtype=self.dtype,
-                              padding_factor=self.padding_factor,
-                              dealias_direct=self.dealias_direct,
-                              coordinates=self.coors.coordinates)
+        d = dict(quad=self.quad,
+                 domain=self.domain,
+                 dtype=self.dtype,
+                 padding_factor=self.padding_factor,
+                 dealias_direct=self.dealias_direct,
+                 coordinates=self.coors.coordinates)
+        if hasattr(self.bc, 'bc'):
+            d['bc'] = tuple(self.bc.bc)
+        if hasattr(self, '_scaled'):
+            d['scaled'] = self._scaled
+        for key in ('alpha', 'beta', 'mean'):
+            if hasattr(self, key):
+                d[key] = object.__getattribute__(self, key)
+        d.update(kwargs)
+        return self.__class__(self.N, **d)
 
     def get_adaptive(self, fun=None, reltol=1e-12, abstol=1e-15):
         """Return space (otherwise as self) with number of quadrature points
@@ -1058,7 +1084,6 @@ class SpectralBase:
             res = T.eval(points, u)
             exact = fx(points)
             energy = np.linalg.norm(res-exact)
-            #print(T.N, energy)
             converged = energy**2 < abstol
             count += 1
 
