@@ -21,9 +21,9 @@ comm = MPI.COMM_WORLD
 __all__ = ('TensorProductSpace', 'VectorSpace', 'TensorSpace',
            'CompositeSpace', 'Convolve')
 
-class CurvilinearTransform(Transform):
-    """Class for performing forward parallel transform using curvilinear
-    coordinates
+
+class ScalarTransform(Transform):
+    """Class for performing the scalar product in parallel
 
     Parameters
     ----------
@@ -95,7 +95,8 @@ class CurvilinearTransform(Transform):
         if input_array is not None:
             self.input_array[...] = input_array
 
-        self.get_measured_input_array()
+        if not self._T.coors.is_cartesian:
+            self.get_measured_input_array()
 
         for i in range(len(self._transfer)):
             self._xfftn[i](**kw)
@@ -110,6 +111,52 @@ class CurvilinearTransform(Transform):
         else:
             return self.output_array
 
+
+class ForwardTransform(ScalarTransform):
+    """Class for performing the forward transform in parallel
+
+    Parameters
+    ----------
+    xfftn : list of serial transform objects
+    transfer : list of global redistribution objects
+    pencil : list of two pencil objects
+        The two pencils represent the input and final output configuration of
+        the distributed global arrays
+    T : TensorProductSpace
+    """
+    def __call__(self, input_array=None, output_array=None, **kw):
+        if input_array is not None:
+            self.input_array[...] = input_array
+
+        if not self._T.coors.is_cartesian:
+            self.get_measured_input_array()
+
+        if len(self._T.get_nonhomogeneous_axes()) > 1:
+            from shenfun import la, TestFunction, TrialFunction, inner
+            assert self._T.dimensions == 2, 'Two inhomogeneous boundary directions only implemented for 2D'
+            u = TrialFunction(self._T)
+            v = TestFunction(self._T)
+            B = inner(u, v)
+            b = inner(v, input_array)
+            sol = la.Solver2D(B)
+            self.output_array[:] = sol(b)
+
+        else:
+
+            for i in range(len(self._transfer)):
+                self._xfftn[i](**kw)
+                arrayA = self._xfftn[i].output_array
+                arrayB = self._xfftn[i+1].input_array
+                self._transfer[i](arrayA, arrayB)
+            self._xfftn[-1](**kw)
+
+        if output_array is not None:
+            output_array[...] = self.output_array
+            return output_array
+        else:
+            return self.output_array
+
+
 class TensorProductSpace(PFFT):
     """Class for multidimensional tensorproductspaces.
 
@@ -297,21 +344,18 @@ class TensorProductSpace(PFFT):
 
             self.pencil[1] = pencilA
 
-            FT = functools.partial(CurvilinearTransform, T=self) if not coordinates is None else Transform
-            #FT = functools.partial(CurvilinearTransform, T=self) if not self.coors.is_cartesian else Transform
-
-            self.forward = FT(
+            self.forward = ForwardTransform(
                 [o.forward for o in self.xfftn],
                 [o.forward for o in self.transfer],
-                self.pencil)
+                self.pencil, self)
             self.backward = Transform(
                 [o.backward for o in self.xfftn[::-1]],
                 [o.backward for o in self.transfer[::-1]],
                 self.pencil[::-1])
-            self.scalar_product = FT(
+            self.scalar_product = ScalarTransform(
                 [o.scalar_product for o in self.xfftn],
                 [o.forward for o in self.transfer],
-                self.pencil)
+                self.pencil, self)
 
         else:
             self.configure_backwards(backward_from_pencil, dtype, kw)
@@ -380,22 +424,22 @@ class TensorProductSpace(PFFT):
 
         self.pencil[1] = pencilA
 
-        FT = functools.partial(CurvilinearTransform, T=self) if not self.coors.is_cartesian else Transform
-
         self.backward = Transform(
             [o.backward for o in self.xfftn],
             [o.forward for o in self.transfer],
             self.pencil)
-        self.forward = FT(
+        self.forward = ForwardTransform(
             [o.forward for o in self.xfftn[::-1]],
             [o.backward for o in self.transfer[::-1]],
-            self.pencil[::-1])
-        self.scalar_product = FT(
+            self.pencil[::-1], self)
+        self.scalar_product = ScalarTransform(
             [o.scalar_product for o in self.xfftn[::-1]],
             [o.forward for o in self.transfer[::-1]],
-            self.pencil[::-1])
+            self.pencil[::-1], self)
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
+        """Return space (otherwise as self) to be used for dealiasing
+        """
         if isinstance(padding_factor, Number):
             padding_factor = (padding_factor,)*len(self)
         elif isinstance(padding_factor, (tuple, list, np.ndarray)):
@@ -414,6 +458,16 @@ class TensorProductSpace(PFFT):
                                   coordinates=self.coors.coordinates)
 
     def get_refined(self, N):
+        """Return space (otherwise as self) refined to new shape
+
+        Parameters
+        ----------
+        N : Number or sequence of integers
+            The global shape of the new space. If N is a number, then
+            refine the global shape by multiplying self's global shape by N.
+            If N is an array, then N is treated as the new global shape of
+            the returned space.
+        """
         if isinstance(N, Number):
             N = N*np.array(self.global_shape())
         elif isinstance(N, (tuple, list, np.ndarray)):
@@ -424,6 +478,28 @@ class TensorProductSpace(PFFT):
                                   dtype=self.dtype(),
                                   coordinates=self.coors.coordinates)
 
+    def get_unplanned(self, tensorproductspace=False, **kwargs):
+        """Return unplanned bases otherwise as self. Or return a new
+        TensorProductSpace from the unplanned bases.
+
+        Parameters
+        ----------
+        tensorproductspace : boolean
+            if True, then return a new TensorProductSpace
+            if False, return only the bases
+        kwargs : keyword arguments
+            Any arguments used in the creation of the different bases.
+
+        Note
+        ----
+        The default behaviour is to return regular spaces with padding_factor=1
+        and dealias_direct=False. This can be overruled in the keyword arguments.
+        """
+        bases = tuple([base.get_unplanned(**kwargs) for base in self.bases])
+        if tensorproductspace:
+            return TensorProductSpace(comm, bases)
+        return bases
+
     def dtype(self, forward_output=False):
         """Return datatype function space is planned for"""
         if hasattr(self, 'forward'):
@@ -461,7 +537,7 @@ class TensorProductSpace(PFFT):
         ab_hat = self.forward(a*b, ab_hat)
         return ab_hat
 
-    def eval(self, points, coefficients, output_array=None, method=2):
+    def eval(self, points, coefficients, output_array=None, method=1):
         """Evaluate Function at points, given expansion coefficients
 
         Parameters
@@ -473,8 +549,8 @@ class TensorProductSpace(PFFT):
         output_array : array, optional
             Return array, function values at points
         method : int, optional
-            Chooses implementation. The default 0 is a low-memory cython
-            version. Using method = 1 leads to a faster cython
+            Chooses implementation. The method 0 is a low-memory cython
+            version. Using method = 1 (default) leads to a faster cython
             implementation that, on the downside, uses more memory.
             The final, method = 2, is a python implementation.
         """
@@ -483,7 +559,8 @@ class TensorProductSpace(PFFT):
         else:
             output_array[:] = 0
         if len(self.get_nonperiodic_axes()) > 1:
-            method = 2
+            method = 1
+        assert self.dimensions < 4, 'eval not implemented (yet) for higher dimensions'
         if method == 0:
             return self._eval_lm_cython(points, coefficients, output_array)
         elif method == 1:
@@ -628,7 +705,7 @@ class TensorProductSpace(PFFT):
         for base in self:
             axis = base.axis
             x = base.map_reference_domain(points[axis])
-            D = base.evaluate_basis_all(x=x)
+            D = base.evaluate_basis_all(x=x, argument=1)
             P.append(D[..., self.local_slice()[axis]])
             if isinstance(base, R2C):
                 r2c = axis
@@ -854,7 +931,7 @@ class TensorProductSpace(PFFT):
         dx = self.coors.get_sqrt_det_g()
         mesh = self.local_mesh(True)
         if dx == 1:
-            return
+            return u
 
         if isinstance(dx, Number):
             u *= dx
@@ -1034,6 +1111,19 @@ class TensorProductSpace(PFFT):
                 break
         return compatible
 
+    def shape(self, forward_output=True):
+        """The local (to each processor) shape of data
+
+        Parameters
+        ----------
+        forward_output : bool, optional
+            Return shape of output array (spectral space) if True, else return
+            shape of input array (physical space)
+        """
+        if forward_output is not True:
+            return self.forward.input_pencil.subshape
+        return self.forward.output_array.shape
+
 
 class CompositeSpace:
     """Class for composite tensorproductspaces.
@@ -1057,7 +1147,7 @@ class CompositeSpace:
     def is_composite_space(self):
         return 1
 
-    def eval(self, points, coefficients, output_array=None, method=0):
+    def eval(self, points, coefficients, output_array=None, method=1):
         """Evaluate Function at points, given expansion coefficients
 
         Parameters
@@ -1068,10 +1158,10 @@ class CompositeSpace:
         output_array : array, optional
             Return array, function values at points
         method : int, optional
-            Chooses implementation. The default 0 is a low-memory cython
-            version. Using method = 1 leads to a faster cython
+            Chooses implementation. Method 0 is a low-memory cython
+            version. Using method = 1 (default) leads to a faster cython
             implementation that, on the downside, uses more memory.
-            The final, method = 2, is a python implementation used only
+            The final, method = 2, is a python implementation used mainly
             for verification.
         """
         if output_array is None:
@@ -1356,6 +1446,14 @@ class TensorSpace(VectorSpace):
     def tensor_rank(self):
         return 2
 
+    def get_refined(self, N):
+        return TensorSpace(self.flatten()[0].get_refined(N))
+
+    def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
+        return TensorSpace(self.flatten()[0].get_dealiased(padding_factor, dealias_direct))
+
+    def get_orthogonal(self):
+        return TensorSpace(self.flatten()[0].get_orthogonal())
 
 class VectorTransform:
 
@@ -1468,7 +1566,7 @@ class BoundaryValues:
     def update_bcs(self, bc=None):
         if bc is not None:
             assert isinstance(bc, (list, tuple))
-            assert len(bc) in (2, 4, 6)
+            assert len(bc) in (2, 3, 4, 6)
             self.bc = list(bc)
             for i, bci in enumerate(bc):
                 if isinstance(bci, (Number, sp.Expr, np.ndarray)):
@@ -1500,7 +1598,7 @@ class BoundaryValues:
         To modify boundary conditions on the fly, modify first self.bc and then
         call this function. The boundary condition can then be applied as before.
         """
-        from shenfun.utilities import split
+        #from shenfun.utilities import split
         from shenfun import project
 
         self.axis = this_base.axis
@@ -1539,7 +1637,7 @@ class BoundaryValues:
 
             b = Array(T)
             s = T.local_slice(False)[self.axis]
-            num_bcs = len(self.bc) - np.count_nonzero(np.array(self.bc) == None)
+            num_bcs = len(self.bc) - np.count_nonzero(np.array(self.bc) is None)
 
             for j, bci in enumerate(self.bc):
                 if isinstance(bci, sp.Expr):
@@ -1576,7 +1674,8 @@ class BoundaryValues:
                 else:
                     raise NotImplementedError
 
-                if len(T.get_nonhomogeneous_axes()) == 1:
+            if len(T.get_nonhomogeneous_axes()) == 1:
+                for j, bci in enumerate(self.bc):
                     from .spectralbase import FuncWrap
 
                     if number_of_bases_after_this == 0:
@@ -1588,8 +1687,7 @@ class BoundaryValues:
                         u_hat = T.forward._xfftn[i].input_array
                         fun = FuncWrap(lambda x, y: x, u_hat, u_hat) # Do-nothing
                         fwd = Transform(list(T.forward._xfftn[:i])+[fun],
-                                 T.forward._transfer[:i],
-                                 [[], []])
+                                        T.forward._transfer[:i], [[], []])
                         b_hat = fwd(b).copy()
 
                     # Now b_hat contains the correct slices in slm1 and slm2
@@ -1599,13 +1697,77 @@ class BoundaryValues:
                         self.bcs[j] = b_hat[this_base.si[-1]].copy()
                     else:
                         self.bcs[j] = b_hat[this_base.si[-(len(self.bc))+j]].copy()
-                    b_hat = T.forward(b).copy()
+
+                b_hat = T.forward(b).copy()
+                for j, bci in enumerate(self.bc):
                     if num_bcs == 1:
                         self.bcs_final[j] = b_hat[this_base.si[-1]].copy()
                     else:
                         self.bcs_final[j] = b_hat[this_base.si[-(len(self.bc))+j]].copy()
 
-            if len(T.get_nonhomogeneous_axes()) == 2:
+            #msdict = split(T.coors.sg)
+            #assert len(msdict) == 1
+            #_c = msdict[0]['coeff']
+            #if len(T.get_nonhomogeneous_axes()) == 1:
+            #    if number_of_bases_after_this == 0:
+            #        # Dirichlet base is the first to be transformed
+            #        b_hat = b
+
+            #    elif number_of_bases_after_this == 1:
+            #        T.forward._xfftn[0].input_array[...] = b*_c
+
+            #        T.forward._xfftn[0]()
+            #        arrayA = T.forward._xfftn[0].output_array
+            #        arrayB = T.forward._xfftn[1].input_array
+            #        T.forward._transfer[0](arrayA, arrayB)
+            #        b_hat = arrayB.copy()
+
+            #    elif number_of_bases_after_this == 2:
+
+            #        T.forward._xfftn[0].input_array[...] = b*_c
+            #        T.forward._xfftn[0]()
+            #        arrayA = T.forward._xfftn[0].output_array
+            #        arrayB = T.forward._xfftn[1].input_array
+            #        T.forward._transfer[0](arrayA, arrayB)
+
+            #        T.forward._xfftn[1]()
+            #        arrayA = T.forward._xfftn[1].output_array
+            #        arrayB = T.forward._xfftn[2].input_array
+            #        T.forward._transfer[1](arrayA, arrayB)
+            #        b_hat = arrayB.copy()
+
+            #    # Now b_hat contains the correct slices in slm1 and slm2
+            #    # These are the values to use on intermediate steps. If for example the Dirichlet space is squeezed between two Fourier spaces
+            #    for i in range(len(self.bc)):
+            #        if self.bcs[i] is None:
+            #            continue
+            #        if num_bcs == 1:
+            #            self.bcs[i] = b_hat[this_base.si[-1]].copy()
+            #        else:
+            #            self.bcs[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
+
+            #    # Final (the values to set on fully transformed functions)
+
+            #    T.forward._xfftn[0].input_array[...] = b*_c
+
+            #    for i in range(len(T.forward._transfer)):
+            #        T.forward._xfftn[i]()
+            #        arrayA = T.forward._xfftn[i].output_array
+            #        arrayB = T.forward._xfftn[i+1].input_array
+            #        T.forward._transfer[i](arrayA, arrayB)
+
+            #    T.forward._xfftn[-1]()
+            #    b_hat = T.forward._xfftn[-1].output_array
+            #    for i in range(len(self.bc)):
+            #        if self.bcs[i] is None:
+            #            continue
+            #        elif num_bcs == 1:
+            #            self.bcs_final[i] = b_hat[this_base.si[-1]].copy()
+            #        else:
+            #            self.bcs_final[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
+
+
+            elif len(T.get_nonhomogeneous_axes()) == 2:
                 assert len(T.bases) == 2, 'Only implemented for 2D'
                 bases = []
                 for axis, base in enumerate(T.bases):
@@ -1615,6 +1777,7 @@ class BoundaryValues:
                 other_base = bases[0]
                 ua = Array(other_base)
                 bc_this = this_base.bc.bc.copy()
+                bc_other = other_base.bc.bc.copy()
                 df = 2./(other_base.domain[1]-other_base.domain[0])
                 for i in range(2): # x = -1 and then x = 1
                     bcj = bc_this[i]
@@ -1632,9 +1795,10 @@ class BoundaryValues:
                                 s = bcj.subs(xx, xj)
                             elif j == 1 and other_base.boundary_condition() == 'UpperDirichletNeumann':
                                 s = bcj.diff(xx, 1).subs(xx, xj)/df
+                            elif other_base.boundary_condition() == 'Neumann':
+                                s = bcj.diff(xx, 1).subs(xx, xj)/df
                             else:
                                 s = bcj.subs(xx, xj)
-
                             other_base.bc.bc[j] = s
                             other_base.bc.bcs[j] = s
                             other_base.bc.bcs_final[j] = s
@@ -1656,7 +1820,7 @@ class BoundaryValues:
             Containing correct boundary values of Function
         """
         B = self.base.get_bc_basis()
-        M = B.coefficient_matrix().T
+        M = (B.stencil_matrix().T).astype(float)
         sl = B.slice()
         ds = uh.shape[self.base.axis] - B.N
         if ds > 0: # uh is padded, but B is not. Boundary dofs are in padded locations.
@@ -1728,59 +1892,3 @@ class BoundaryValues:
             elif isinstance(bc, sp.Expr):
                 return True
         return False
-
-def some_basic_tests():
-    import pyfftw             #pylint: disable=import-error
-
-    N = 8
-    K0 = C2C(N)
-    K1 = C2C(N)
-    K2 = C2C(N)
-    K3 = R2C(N)
-    T = TensorProductSpace(comm, (K0, K1, K2, K3))
-
-    # Create data on rank 0 for testing
-    if comm.Get_rank() == 0:
-        f_g = np.random.random(T.shape())
-        f_g_hat = pyfftw.interfaces.numpy_fft.rfftn(f_g, axes=(0, 1, 2, 3))
-    else:
-        f_g = np.zeros(T.shape())
-        f_g_hat = np.zeros(T.shape(), dtype=complex)
-
-    # Distribute test data to all ranks
-    comm.Bcast(f_g, root=0)
-    comm.Bcast(f_g_hat, root=0)
-
-    # Create a function in real space to hold the test data
-    fj = Array(T)
-    fj[:] = f_g[T.local_slice(False)]
-
-    # Perform forward transformation
-    f_hat = T.forward(fj)
-
-    assert np.allclose(f_g_hat[T.local_slice(True)], f_hat*N**4)
-
-    # Perform backward transformation
-    fj2 = Array(T)
-    fj2 = T.backward(f_hat)
-
-    assert np.allclose(fj, fj2)
-
-    f_hat = T.scalar_product(fj)
-
-    # Padding
-    # Needs new instances of bases because arrays have new sizes
-    Kp0 = C2C(N, padding_factor=1.5)
-    Kp1 = C2C(N, padding_factor=1.5)
-    Kp2 = C2C(N, padding_factor=1.5)
-    Kp3 = R2C(N, padding_factor=1.5)
-    Tp = TensorProductSpace(comm, (Kp0, Kp1, Kp2, Kp3))
-
-    f_g_pad = Tp.backward(f_hat)
-    f_hat2 = Tp.forward(f_g_pad)
-
-    assert np.allclose(f_hat2, f_hat)
-
-
-if __name__ == '__main__':
-    some_basic_tests()
